@@ -79,9 +79,14 @@ def form_interfaces(context):
 
     """
     dutils = getallutils(IDexterityFTI)
-    behavior = 'seantis.reservation.interfaces.IReservationFormSet'
+    behaviors = set((
+        'seantis.reservation.interfaces.IReservationFormSet',
+        'seantis.reservation.interfaces.IReservationManagerFormSet'
+    ))
     interfaces = [
-        (u.title, u.lookupSchema()) for u in dutils if behavior in u.behaviors
+        (
+            u.title, u.lookupSchema()
+        ) for u in dutils if behaviors & set(u.behaviors)
     ]
 
     def get_term(item):
@@ -241,6 +246,10 @@ class IReservationFormSet(Interface):
     """ Marks interface as usable for sub-forms in a resource object. """
 
 
+class IReservationManagerFormSet(IReservationFormSet):
+    """ Same as IReservationFormSet but only available to managers. """
+
+
 class IResourceAllocationDefaults(form.Schema):
 
     quota = schema.Int(
@@ -296,8 +305,8 @@ class IResourceAllocationDefaults(form.Schema):
 
     @invariant
     def isValidQuota(Allocation):
-        if not (1 <= Allocation.quota and Allocation.quota <= 100):
-            raise Invalid(_(u'Quota must be between 1 and 100'))
+        if not (1 <= Allocation.quota and Allocation.quota <= 1000):
+            raise Invalid(_(u'Quota must be between 1 and 1000'))
 
     @invariant
     def isValidQuotaLimit(Allocation):
@@ -520,7 +529,7 @@ class IAllocation(IResourceAllocationDefaults):
         title=_(u'Separately reservable'),
         description=_(
             u'If checked parts of the recurrance may be reserved. '
-            u'If not checkd the recurrance must be reserved as a whole.'
+            u'If not checked the recurrance must be reserved as a whole.'
         ),
         required=False,
         default=False
@@ -573,7 +582,8 @@ template_variables = _(
     u'%(reservation_mail)s - email of reservee<br>'
     u'%(data)s - formdata associated with the reservation<br>'
     u'%(approval_link)s - link to the approval view<br>'
-    u'%(denial_link)s - link to the denial view'
+    u'%(denial_link)s - link to the denial view<br>'
+    u'%(cancel_link)s - link to the cancel view'
 )
 
 template_revoke_variables = template_variables + _(
@@ -594,6 +604,20 @@ class IEmailTemplate(form.Schema):
         source=plone_languages
     )
 
+    reservation_made_subject = schema.TextLine(
+        title=_(u'Email Subject for Reservation Autoapproved'),
+        description=_(u'Sent to <b>managers</b> when a reservation is '
+                      u'automatically approved. '
+                      u'May contain the template variables listed below.'),
+        default=templates['reservation_made'].get_subject('en')
+    )
+
+    reservation_made_content = schema.Text(
+        title=_(u'Email Text for Reservation Autoapproved'),
+        description=template_variables,
+        default=templates['reservation_made'].get_body('en')
+    )
+
     reservation_pending_subject = schema.TextLine(
         title=_(u'Email Subject for Reservation Pending'),
         description=_(
@@ -708,6 +732,16 @@ class IReservation(Interface):
     )
 
 
+class IReservationIdForm(Interface):
+    """ Describes a form with a hidden reservation-id field. Use with
+    seantis.reservation.reserve.ReservationIdForm. """
+
+    reservation = schema.Text(
+        title=_(u'Reservation'),
+        required=False
+    )
+
+
 class IGroupReservation(Interface):
     """ A reservation of an allocation group. """
 
@@ -728,14 +762,9 @@ class IGroupReservation(Interface):
     )
 
 
-class IRevokeReservation(Interface):
+class IRevokeReservation(IReservationIdForm):
     """ For the reservation revocation form. """
 
-    reservation = schema.Text(
-        title=_(u'Reservation'),
-        required=False
-    )
-
     reason = schema.Text(
         title=_(u'Reason'),
         description=_(
@@ -747,15 +776,6 @@ class IRevokeReservation(Interface):
     )
 
 
-class IApproveReservation(Interface):
-    """ For the reservation approval form. """
-
-    reservation = schema.Text(
-        title=_(u'Reservation'),
-        required=False
-    )
-
-
 class IResourceViewedEvent(Interface):
     """ Event triggered when a seantis.reservation resource is viewed. Pretty
     useful if you need a hook which is guaranteed to be triggered on a plone
