@@ -98,7 +98,7 @@ def additional_data_dictionary(data, fti):
     """
 
     result = dict()
-    used_data = dict([(k, v) for k, v in data.items() if v])
+    used_data = dict([(k, v) for k, v in data.items() if v is not None])
 
     def values(iface, ifacekey):
         for key, value in used_data.items():
@@ -262,26 +262,25 @@ def compare_link(resources):
     return link.rstrip('&')
 
 
-def export_link(source, extension, context, request, resources):
+def export_link(context, request, resources):
     """Builds the reservations excel export link for the
     given list of resources.
 
     """
 
-    # ensure that the view may be called
-    viewname = 'resource_export.' + extension
-    if not _exposure().for_views(context, request)(viewname):
+    if not resources:
         return ''
 
-    uuids = map(string_uuid, resources)
-
-    url = context.absolute_url()
-    url += '/resource_export.' + extension + '?source=' + source
-    url += '&uuid=' + '&uuid='.join(uuids)
-    url += '&lang=' + get_current_site_language()
-
-    return url
+    if not _exposure().for_views(context, request)('reservation-exports'):
+        return ''
 
+    return ''.join((
+        context.absolute_url(),
+        '/reservation-exports?uuid=',
+        '&uuid='.join(
+            map(string_uuid, resources)
+        )
+    ))
 
 def monthly_report_link(context, request, resources):
     """Builds the monthly report link given the list of the resources
@@ -473,10 +472,17 @@ def uuid_query(uuid):
         uuid[:8], uuid[8:12], uuid[12:16], uuid[16:20], uuid[20:]]))
 
 
-def get_resource_by_uuid(uuid):
+def get_resource_by_uuid(
+    uuid, ensure_portal_type='seantis.reservation.resource'
+):
     """Returns the zodb object with the given uuid."""
     catalog = getToolByName(getSite(), 'portal_catalog')
-    results = catalog(UID=uuid_query(uuid))
+
+    if ensure_portal_type:
+        results = catalog(UID=uuid_query(uuid), portal_type=ensure_portal_type)
+    else:
+        results = catalog(UID=uuid_query(uuid))
+
     return len(results) == 1 and results[0] or None
 
 
@@ -519,6 +525,9 @@ class UserFormDataEncoder(json.JSONEncoder):
 
     def default(self, obj):
 
+        if isinstance(obj, set):
+            return list(obj)
+
         if isinstance(obj, datetime):
             return u'__datetime__@%s' % isodate.datetime_isoformat(obj)
 
@@ -659,6 +668,11 @@ def flatten(l):
             yield el
 
 
+def pack(v):
+    """Puts v in a list if not already an iterator."""
+    return [v] if not hasattr(v, '__iter__') else v
+
+
 def pairs(l):
     """Takes any list and returns pairs:
     ((a,b),(c,d)) => ((a,b),(c,d))
@@ -672,7 +686,7 @@ def pairs(l):
 
 
 def pairwise(iterable):
-    """Almost like paris, but not quite:
+    """Almost like pairs, but not quite:
     "s -> (s0,s1), (s1,s2), (s2, s3), ..."
     """
     a, b = tee(iterable)
@@ -880,6 +894,31 @@ def shift_day(stdday):
     return abs(0 - (stdday + 1) % 7)
 
 
+def whole_day(start, end):
+    """Returns true if the given start, end range should be considered
+    a whole-day range. This is so if the start time is 0:00:00 and the end
+    time either 0:59:59 or 0:00:00 and if there is at least a diff
+    erence of 23h 59m 59s / 86399 seconds between them.
+
+    This is relevant for the calendar-display for now. This might very well be
+    replaced again in the future when we introduce timezones.
+
+    """
+
+    assert start <= end, "The end needs to be equal or greater than the start"
+
+    if (start.hour, start.minute, start.second) != (0, 0, 0):
+        return False
+
+    if (end.hour, end.minute, end.second) not in ((0, 0, 0), (23, 59, 59)):
+        return False
+
+    if (end - start).total_seconds() < 86399:
+        return False
+
+    return True
+
+
 def context_path(context):
     """Returns the physical path for brains and objects alike."""
 
@@ -889,6 +928,13 @@ def context_path(context):
         return context.getPhysicalPath()
 
 
+def portal_type_in_site(portal_type):
+    catalog = getToolByName(getSite(), 'portal_catalog')
+    results = catalog(portal_type=portal_type)
+
+    return results
+
+
 def portal_type_in_context(context, portal_type, depth=1):
     """Returns the given portal types _within_ the current context."""
 
@@ -982,6 +1028,101 @@ def display_date(start, end):
             + end.strftime('%d.%m.%Y %H:%M')
 
 
+class United(object):
+    """ Puts items added through 'append' into the same group as the last
+    item which was appended, as long as the matchfn which is passed the last
+    and the current item returns true.
+
+    e.g.
+
+    united = United(lambda last, current: last == current)
+
+    united.append(1)
+    -> united.groups => [[1]]
+
+    united.append(1)
+    -> united.groups => [[1, 1]]
+
+    united.append(2)
+    -> united.groups => [[1, 1], [2]]
+
+    united.append(1)
+    -> united.groups => [[1, 1], [2], [1]]
+
+    """
+
+    def __init__(self, matchfn):
+        self.matchfn = matchfn
+        self.groups = []
+        self.new_group()
+
+    def new_group(self):
+        self.groups.append([])
+
+    @property
+    def current_group(self):
+        return self.groups[-1]
+
+    def append(self, item):
+        if not self.current_group:
+            self.current_group.append(item)
+        else:
+            if not self.matchfn(self.current_group[-1], item):
+                self.new_group()
+
+            self.current_group.append(item)
+
+
+def unite(iterator, matchfn):
+    """ Iterates through the given records and groups the records if two
+    records passed to the match function result in True.
+
+    e.g.
+
+    unite([1, 1, 1, 2, 2], lambda last, current: last == current)
+
+    -> [[1, 1, 1], [2, 2]]
+
+    Note that for this to work the records need to be sorted, not unlike
+    when using the collection's group function. See 'United' doc above to
+    learn why.
+    """
+
+    united = United(matchfn)
+
+    for item in iterator:
+        united.append(item)
+
+    return united.groups
+
+
+def unite_dates(dates):
+    """ Takes a list of start / end date tuples and compacts it by merging
+    dates which can be considered an unbroken range.
+
+    So given a list like this:
+
+        [
+            (01.01.2012 00:00, 02.01.2012 00:00),
+            (02.01.2012 00:00, 03.01.2012 00:00),
+            (05.01.2012 00:00, 06.01.2012 00:00)
+        ]
+
+    (Where the dates are actually datetime instances)
+
+    We should get this result:
+
+        [
+            (01.01.2012 00:00, 03.01.2012 00:00),
+            (05.01.2012 00:00, 06.01.2012 00:00)
+        ]
+
+    """
+    sorted_dates = sorted(dates, key=lambda pair: pair[0])
+    for group in unite(sorted_dates, lambda this, next: this[1] == next[0]):
+        yield group[0][0], group[-1][1]
+
+
 class cached_property(object):
     """A read-only @property that is only evaluated once. The value is cached
     on the object itself rather than the function or class; this should prevent
